from typing import Any, TypeVar

from mcp.types import TextContent
from pydantic import BaseModel, RootModel

T = TypeVar("T")


def dump_json_list(items: list[T], indent: int | None = 2) -> str:
    return RootModel(items).model_dump_json(indent=indent, exclude_none=True)


def dump_list(items: list[T]) -> dict[str, Any]:
    return RootModel(items).model_dump(exclude_none=True)


def prepare_text_content(
    obj: dict[str, Any] | list[Any] | BaseModel | str, indent: int | None = None
) -> TextContent:
    if isinstance(obj, list):
        content = dump_json_list(obj, indent=indent)
    elif isinstance(obj, dict):
        content = RootModel(obj).model_dump_json(indent=indent)
    elif isinstance(obj, BaseModel):
        content = obj.model_dump_json(indent=indent, exclude_none=True)
    elif isinstance(obj, str):
        content = obj
    else:
        raise TypeError(
            f"Unsupported type {type(obj)} for content preparation. "
            "Expected list, BaseModel, or str."
        )

    return TextContent(
        type="text",
        text=content,
    )



from typing import Annotated

from pydantic import Field

IssueID = Annotated[
    str,
    Field(description="Issue ID in the format '<project>-<id>', like 'SOMEPROJECT-1'"),
]

QueueID = Annotated[
    str,
    Field(
        description="Queue (Project ID) to search in, like 'SOMEPROJECT'",
    ),
]

IssueIDs = Annotated[
    list[str],
    Field(
        description="Multiple Issue IDs. Each issue id is in the format '<project>-<id>', like 'SOMEPROJECT-1'"
    ),
]



from typing import Any

from aiocache import cached

from mcp_tracker.tracker.proto.fields import GlobalDataProtocolWrap
from mcp_tracker.tracker.proto.issues import IssueProtocolWrap
from mcp_tracker.tracker.proto.queues import QueuesProtocolWrap
from mcp_tracker.tracker.proto.types.fields import GlobalField, LocalField
from mcp_tracker.tracker.proto.types.issues import (
    Issue,
    IssueComment,
    IssueLink,
    Worklog,
)
from mcp_tracker.tracker.proto.types.queues import Queue
from mcp_tracker.tracker.proto.types.statuses import Status


def make_cached_protocols(
    cache_config: dict[str, Any],
) -> tuple[
    type[QueuesProtocolWrap], type[IssueProtocolWrap], type[GlobalDataProtocolWrap]
]:
    class CachingQueuesProtocol(QueuesProtocolWrap):
        @cached(**cache_config)
        async def queues_list(self, per_page: int = 100, page: int = 1) -> list[Queue]:
            return await self._original.queues_list(per_page=per_page, page=page)

        @cached(**cache_config)
        async def queues_get_local_fields(self, queue_id: str) -> list[LocalField]:
            return await self._original.queues_get_local_fields(queue_id)

    class CachingIssuesProtocol(IssueProtocolWrap):
        @cached(**cache_config)
        async def issue_get(self, issue_id: str) -> Issue | None:
            return await self._original.issue_get(issue_id)

        @cached(**cache_config)
        async def issues_get_links(self, issue_id: str) -> list[IssueLink] | None:
            return await self._original.issues_get_links(issue_id)

        @cached(**cache_config)
        async def issue_get_comments(self, issue_id: str) -> list[IssueComment] | None:
            return await self._original.issue_get_comments(issue_id)

        @cached(**cache_config)
        async def issues_find(
            self,
            query: str,
            *,
            per_page: int = 15,
            page: int = 1,
        ) -> list[Issue]:
            return await self._original.issues_find(
                query=query,
                per_page=per_page,
                page=page,
            )

        @cached(**cache_config)
        async def issue_get_worklogs(self, issue_id: str) -> list[Worklog] | None:
            return await self._original.issue_get_worklogs(issue_id)

    class CachingFieldsProtocol(GlobalDataProtocolWrap):
        @cached(**cache_config)
        async def get_global_fields(self) -> list[GlobalField]:
            return await self._original.get_global_fields()

        @cached(**cache_config)
        async def get_statuses(self) -> list[Status]:
            return await self._original.get_statuses()

    return CachingQueuesProtocol, CachingIssuesProtocol, CachingFieldsProtocol



from dataclasses import dataclass

from mcp_tracker.tracker.proto.fields import GlobalDataProtocol
from mcp_tracker.tracker.proto.issues import IssueProtocol
from mcp_tracker.tracker.proto.queues import QueuesProtocol


@dataclass
class AppContext:
    queues: QueuesProtocol
    issues: IssueProtocol
    fields: GlobalDataProtocol



from typing import Protocol

from .types.fields import LocalField
from .types.queues import Queue


class QueuesProtocol(Protocol):
    async def queues_list(self, per_page: int = 100, page: int = 1) -> list[Queue]: ...

    async def queues_get_local_fields(self, queue_id: str) -> list[LocalField]: ...


class QueuesProtocolWrap(QueuesProtocol):
    def __init__(self, original: QueuesProtocol):
        self._original = original



from typing import Literal

from pydantic import BaseModel, Field

from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity


class Status(BaseTrackerEntity):
    version: int = Field(description="Status version")
    key: str = Field(description="Status key")
    name: str = Field(description="Displayed status name")
    description: str | None = Field(default=None, description="Status description")
    order: int = Field(description="Status order")
    type: Literal["new", "inProgress", "paused", "done", "cancelled"] = Field(
        description="Status type"
    )



from mcp_tracker.mcp.server import mcp, settings


def main() -> None:
    """Main entry point for the yandex-tracker-mcp command."""
    mcp.run(transport=settings.transport)


if __name__ == "__main__":
    main()



import datetime

from pydantic import AliasChoices, Field

from mcp_tracker.tracker.proto.types.refs import UserReference


class CreatedUpdatedMixin:
    created_at: datetime.datetime | None = Field(
        None, validation_alias=AliasChoices("createdAt", "created_at")
    )
    updated_at: datetime.datetime | None = Field(
        None, validation_alias=AliasChoices("updatedAt", "updated_at")
    )
    created_by: UserReference | None = Field(
        None, validation_alias=AliasChoices("createdBy", "created_by")
    )
    updated_by: UserReference | None = Field(
        None, validation_alias=AliasChoices("updatedBy", "updated_by")
    )



from typing import Any

from pydantic import AliasChoices, BaseModel, Field

from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity


class BaseReference(BaseTrackerEntity):
    id: Any | None = None


class IssueTypeReference(BaseReference):
    key: str | None = None
    display: str | None = None


class PriorityReference(BaseReference):
    key: str | None = None
    display: str | None = None


class QueueReference(BaseReference):
    key: str | None = None
    display: str | None = None


class StatusReference(BaseModel):
    key: str | None = None
    display: str | None = None


class UserReference(BaseReference):
    display: str | None = None
    cloud_uid: str | None = Field(
        None, validation_alias=AliasChoices("cloudUid", "cloud_uid")
    )
    passport_uid: int | None = Field(
        None, validation_alias=AliasChoices("passportUid", "passport_uid")
    )


class IssueReference(BaseReference):
    key: str | None = None
    display: str | None = None


class ComponentReference(BaseReference):
    display: str | None = None
